#define MODULE
#define __KERNEL__
/* Saint Jude, Linux Kernel Module.
 * Verion: 0.20
 * 
 * July 28, 2001 
 *
 *
 *    Copyright (C) 2001  Timothy Lalwess (lawless@netdoor.com)
 *
 *    This program is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU General Public License as
 *    published by the Free Software Foundation; either version 2 of
 *    the License, or (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be
 *    useful, but WITHOUT ANY WARRANTY; without even the implied
 *    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *    PURPOSE.  See the GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 *    USA.
 *      
 *
 * 
 */

#include <linux/modversions.h>
#include <linux/sys.h>
#include <linux/sched.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <asm/uaccess.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <asm/segment.h>
#include <asm/unistd.h>
#include <linux/mm.h>
#include <linux/smp.h>
#include <linux/signal.h>
#include <linux/slab.h>
#include <asm/unistd.h>
#include <asm/current.h>
#include <sys/syscall.h>
#include <asm/errno.h>
#include <asm/ptrace.h>
#include <asm/pgtable.h>
#include "StJude_lkm.h"
#include "StJude_string_util.h"


extern SJ_PRIV *sj_priv_hash[SJ_PRIV_HASH];
extern SJ_MEMORY *sj_argv_memory_hash[SJ_MEMORY_HASH];

extern SJ_RULEBASE sj_rulebase[];


#if defined(USE_OLD_EXECVE)
int sjexecve;
#endif


int
init_module(void)
{
    struct task_struct *p;
    struct task_struct *inittask;

#if defined(CLOAK)
    extern struct module **eml;
    extern struct module *iml;
#endif

#if defined(USE_OLD_EXECVE)
    int sjlandmine;
#endif

    struct sj_argv_memory *memory;
    int             i;

    /*
     * Our memory is not cleared for us. We need to init our hashes so that
     * each hash is like null. 
     */

    for (i = 0; i < SJ_PRIV_HASH; i++)
	sj_priv_hash[i] = NULL;

    for (i = 0; i < SJ_MEMORY_HASH; i++)
	sj_argv_memory_hash[i] = NULL;

#if defined(USE_OLD_EXECVE)
    sjexecve = NR_syscalls - 1 - SJ_MINES_FORWARD - (current->pid %5);
#endif

    sj_print("--=={ Loading %s }\n", VERSION);

    /*
     * Now to "Pardon" all running processes. We are doing this (kludge)
     * because we need some starting point. We could do the hard thing and
     * "figure" out the initial restrictions for our processes.
     * 
     * This is a TODO thing. If we do do this, then we can detect network
     * based root compromises. Wouldn't that be fun?
     * 
     * If we fail here, we abort -- leaving our kernel nice and working.
     * 
     */
#ifdef __SMP__

    /* Initilize our Spinlocks to Unlocked... */
    rwlock_init(&sj_running);
    rwlock_init(&priv_lock);
    rwlock_init(&argv_memory_lock);

    /* Lock the tasklist while we muck with it. */
    read_lock(&tasklist_lock);
#endif

    p = (inittask = &(init_task_union.task))->next_task;
    do {
	char            null_argv[MAX_KEY_ELEMENTS + 1][BUFFSIZE] =
	    NULL_ARGV;
	if (!p)
	    break;		/*
				 * healthy paranoia 
				 */

	memory = create_argv_memory(p->pid, null_argv);
	if (!memory) {
	    sj_print("(StJude) Unable to Init StJude. Aborting.\n");
	    return 1;
	}

	if (!(p->uid && p->euid && p->suid)) {
	    struct sj_priv *priv;

	    priv = create_priv_record(p->pid, 1);
	    if (!priv) {
		sj_print("(StJude): Unable to Init StJude. Aborting.\n");
		return 1;
	    }
	}
	p = p->next_task;
    } while (p && p != inittask);

#ifdef __SMP__
    read_unlock(&tasklist_lock);
#endif

    orig_do_execve = sys_call_table[__NR_execve];

#if defined(USE_OLD_EXECVE)
    /*
     * Kinda dirty, I know. The sys_call_table[0] is a dummy function, it
     * replaces the old setup syscall. We are looking for an empty slot. This 
     * dummy function fills those slots. 
     *
     * Reference: Complete Linux Loadable Kernel Modules by Pragmatic/THC.
     */

    while (sjexecve != 0 && sys_call_table[sjexecve] != sys_call_table[0])
	sjexecve--;

    if (sjexecve) {
	sys_call_table[sjexecve] = orig_do_execve;
	sys_call_table[__NR_execve] = (void *) sj_do_execve;

        /* Lay the Mines ... */
        for (sjlandmine = sjexecve - 1; 
          (sjlandmine > sjexecve - SJ_MINES_BACK - 1) &&
            sjlandmine > 0; sjlandmine--)
             if ( sys_call_table[sjlandmine] == sys_call_table[0] )
                         sys_call_table[sjlandmine] = (void *) sj_landmine; 
        

        for (sjlandmine = sjexecve + 1; 
          (sjlandmine < sjexecve + SJ_MINES_FORWARD + 1) &&
            sjlandmine < NR_syscalls; sjlandmine++)
             if ( sys_call_table[sjlandmine] == sys_call_table[0] )
                         sys_call_table[sjlandmine] = (void *) sj_landmine; 
    } else {
	printk("<1>Unable to Load StJude.\n");
	return 1;
    }

#else
	sys_call_table[__NR_execve] = (void *) sj_do_execve;
#endif

    (void *) orig_fork = sys_call_table[__NR_fork];
    (void *) orig_vfork = sys_call_table[__NR_vfork];
    (void *) orig_clone = sys_call_table[__NR_clone];
    (void *) orig_exit = sys_call_table[__NR_exit];
    (void *) orig_setuid = sys_call_table[__NR_setuid];
    (void *) orig_setreuid = sys_call_table[__NR_setreuid];

#if defined(USE_STMICHAEL) && !defined(LEARNING)
    (void *) orig_init_module = sys_call_table[__NR_init_module]; 
#endif

#if !defined(LEARNING) && !defined(NOSEAL)
    (void *) orig_delete_module = sys_call_table[__NR_delete_module];
#endif

#if !defined(LEARNING)
#if  defined(USE_STMICHAEL)
    sys_call_table[__NR_init_module] = (void *) sj_init_module;
#if  defined(CLOAK)
    (void *) orig_create_module = sys_call_table[__NR_create_module];
    sys_call_table[__NR_create_module] = (void *) sj_create_module;
#endif
#else
#if !defined(NOSEAL)
    (void *) orig_create_module = sys_call_table[__NR_create_module];
    sys_call_table[__NR_create_module] = (void *) sj_create_module;
#endif // NOSEAL
#endif // of Else for StMichael
    sys_call_table[__NR_delete_module] = (void *) sj_delete_module;
#endif

    sys_call_table[__NR_setuid] = (void *) sj_setuid;
    sys_call_table[__NR_setreuid] = (void *) sj_setreuid;
    sys_call_table[__NR_fork] = (void *) sj_fork;
    sys_call_table[__NR_vfork] = (void *) sj_vfork;
    sys_call_table[__NR_clone] = (void *) sj_clone;
    sys_call_table[__NR_exit] = (void *) sj_exit;

#if defined(USE_STMICHAEL) && !defined(LEARNING)

#if defined(USE_CHECKSUM)
    (void *) syscall_reboot = sys_call_table[__NR_reboot];
    (void *) syscall_sync = sys_call_table[__NR_sync];
#endif


#endif

#if defined(CLOAK) && !defined(LEARNING)
    {
       struct module *m, **v;

       for (m = THIS_MODULE; m->next != NULL; m = m->next );
       v = (struct module **) (++m);
       *v = (THIS_MODULE)->next;

       (THIS_MODULE)->nsyms = 0;
       eml = v;
       iml = NULL;
    }

#endif
    

#ifdef FSCHECK
#ifdef __SMP__
   rwlock_init(&fscheck_lock);
   write_lock(&fscheck_lock);
#endif
   (void *) sj_open = sys_call_table[__NR_open];
   (void *) sj_close = sys_call_table[__NR_close];
   init_fscheck_records();
#ifdef __SMP__
   write_unlock(&fscheck_lock);
#endif
#endif

#if defined(USE_STMICHAEL) && !defined(LEARNING)
        sj_integrity_check_init();
	sj_timer_init();
#endif

#ifdef ROKMEM
   sj_kemem_ro();
#endif


sj_print("--=={ %s Loaded }\n", VERSION);

#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,4,8)) 
 sj_print("NOTICE: StJude has Not been Verified to Work on This Kernel.\n");
#endif


    return 0;
}

void
cleanup_module(void)
{
    /*
     * We should free our memory.. 
     */

  
    int             i;
   

sj_print("--=={ Unloading %s }\n", VERSION);
#if defined(LEARNING) || ( defined(NOSEAL) && !defined(CLOAK) )

    sys_call_table[__NR_setuid] = (void *) orig_setuid;
    sys_call_table[__NR_setreuid] = (void *) orig_setreuid;
    sys_call_table[__NR_fork] = (void *) orig_fork;
    sys_call_table[__NR_vfork] = (void *) orig_vfork;
    sys_call_table[__NR_clone] = (void *) orig_clone;
    sys_call_table[__NR_exit] = (void *) orig_exit;
    sys_call_table[__NR_execve] = (void *) orig_do_execve;
#if defined(USE_OLD_EXECVE)
    sys_call_table[sjexecve] = sys_call_table[0];
#endif
     /* 
       We should wait until all calls are finished before ripping the 
       priv and memory out from underneath them, no? 
     */

#ifdef __SMP__
    write_lock(&sj_running);
#endif

    for (i = 0; i < SJ_PRIV_HASH; i++) {
	struct sj_priv *j,
	               *k;

	if (sj_priv_hash[i])
	    for (j = sj_priv_hash[i]; j;) {
		k = j;
		j = j->next;
		kfree(k);
	    }
    }
    
    for (i = 0; i < SJ_MEMORY_HASH; i++) {
	struct sj_argv_memory *j, *k;

	if (sj_argv_memory_hash[i])
	    for (j = sj_argv_memory_hash[i]; j;) {
		k = j;
		j = j->next;
		kfree(k);
	    }
    }



#if defined(USE_OLD_EXECVE)
    for (i = NR_syscalls -1; i > 0; i--) 
          if(sys_call_table[i] == sj_landmine)
              sys_call_table[i] = sys_call_table[0];
#endif
    /* Kinda a mute point, but we do it for ascetics. */

#ifdef __SMP__
    write_unlock(&sj_running);
#endif // __SMP__

#endif

sj_print("--=={ %s Unloaded }\n", VERSION);
}



