#define __KERNEL__
/* Saint Jude, Linux Kernel Module.
 * Verion: 0.20
 * 
 * July 28, 2001 
 *
 *
 *    Copyright (C) 2001  Timothy Lalwess (lawless@netdoor.com)
 *
 *    This program is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU General Public License as
 *    published by the Free Software Foundation; either version 2 of
 *    the License, or (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be
 *    useful, but WITHOUT ANY WARRANTY; without even the implied
 *    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *    PURPOSE.  See the GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 *    USA.
 *      
 *
 * 
 */

#include <linux/modversions.h>
#include <linux/sys.h>
#include <linux/sched.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <asm/uaccess.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <asm/segment.h>
#include <asm/unistd.h>
#include <linux/mm.h>
#include <linux/smp.h>
#include <linux/signal.h>
#include <linux/slab.h>
#include <asm/unistd.h>
#include <asm/current.h>
#include <sys/syscall.h>
#include <asm/errno.h>
#include <asm/ptrace.h>
#include <asm/pgtable.h>
#include <linux/mman.h>
#include "StJude_lkm.h"
#include "StJude_string_util.h"

mm_segment_t old_fs;

void* sj_memset(void* mem,unsigned char value,unsigned int s)
{
    register unsigned int i;
    
    if(!mem || !s) return NULL;
    for(i = 0;i < s;i++) ((unsigned char*)mem)[i] = value;
    return mem;
}

void* sj_memcpy(void* dst,const void* src,unsigned int s)
{
    register unsigned int i;
    
    if(!src || !dst || !s) return NULL;
    for(i = 0;i < s;i++) ((unsigned char*)dst)[i] = ((unsigned char*)src)[i];
    return dst;
}

int sj_memcmp(const void* s1,const void* s2,unsigned int s)
{
    register unsigned int i;
    
    if(!s1 || !s2 || !s) return 1;
    for(i = 0;i < s;i++) {
	if(((unsigned char*)s1)[i] != ((unsigned char*)s2)[i]) return 1;
    }
    return 0;
}

char* sj_strncpy(char* dst,const char* src,unsigned int s)
{
    register unsigned int i;
    
    if(!dst || !src || !s) return NULL;
    sj_memset(dst,0,s);
    for(i = 0;i < s;i++) {
	if(src[i] == '\0') break;
	dst[i] = src[i];
    }
    return dst;
}

inline int sj_strnlen(const char* s1,int max_len)
{
    register int i;

    for (i = 0; i < max_len && s1[i]; i++);
    return i;
}

int sj_strlen(const char* s1)
{
    register unsigned int i;

    if(!s1) return 0;
    for(i = 0;s1[i];i++);
    return i;
}

inline int sj_strncmp(const char* s1,const char* s2,int max_len)
{
    register int i;

    for (i = 0; i < max_len && ( s1[i] != '\0' && s2[i] != '\0') ; i++)
	if (s1[i] != s2[i])
	    return 1;
    return 0;
}

int sj_strcmp(const char* s1,const char* s2)
{
    register unsigned int i;

    if(!s1 || !s2) return 1;
    for(i = 0;(s1[i] != '\0' && s2[i] != '\0');i++) {
	if(s1[i] != s2[i]) return 1;
    }
    return 0;
}

char* sj_substr(char* s,int start,int end)
{
    int i;
    char* buf = NULL;

    if(!(buf = kmalloc(sizeof(char) * (end - start + 1),GFP_KERNEL)))
	return NULL;
    for (i = 0; i < (end - start + 1); i++) buf[i] = '\0';
    for (i = start; i <= end; i++) buf[i] = s[i];
    return buf;
}

int count(char **argv, int max)
{
    int             i = 0;

    if (argv != NULL) {
	for (;;) {
	    char           *p;
	    int             error;

	    error = get_user(p, argv);
	    if (error)
		return error;
	    if (!p)
		break;
	    argv++;
	    if (++i > max)
		return -E2BIG;
	}
    }
    return i;
}

char* sj_strdup(const char* str)
{
    char* ret = NULL;
    unsigned int s = 0;
    
    if(!str) return NULL;
    s = sj_strlen(str) + 1;
    if(!(ret = (char*) kmalloc(s,GFP_KERNEL))) return NULL;
    sj_memset(ret,0,s);
    sj_strncpy(ret,str,s - 1);
    return ret;
}

/*  This is a more secure (I hope...) replacement for printk. It's a function
    local to this module and its address isn't exported to the kernel, so it
    isn't so easy to modify as printk. Unfortunally it uses sys_write() and
    isn't able to log anything... */

int sj_printf(const char* fmt,...)
{
    va_list args;
    char buf[4096];
    static int (*write)(unsigned int,char*,ssize_t);
    static unsigned char init = 0;
	
    if(!fmt) return -1;
    if(!init) {
	write = sys_call_table[SYS_write];
	init = 1;
    }
    sj_memset(buf,0,sizeof(buf));
    va_start(args,fmt);
    vsprintf(buf,fmt,args);
    va_end(args);
    BEGIN_KMEM
    write(1,buf,sj_strlen(buf));
    END_KMEM
    return 0;
}



void * sj_malloc(unsigned long size)
{
   unsigned long memory;
   unsigned long req_size;

   if (size % 1024)
        req_size = size + ( 1024 - size % 1024 );  
   else
   	req_size = size;

   memory =  do_mmap( NULL, 0, req_size, 
   		PROT_READ|PROT_WRITE, MAP_PRIVATE|MAP_ANONYMOUS,0);

   if ( memory > -4096 )
   	return NULL; /* Error.. Don't care which one. */
   else
        return (void *) memory;
   
}

void sj_free( void * memory, unsigned long size)
{
   unsigned long req_size;

   if (size % 1024)
        req_size = size + ( 1024 - size % 1024 );  
   else
   	req_size = size;

#if ((LINUX_VERSION_CODE > KERNEL_VERSION(2,2,0)) &&\
     LINUX_VERSION_CODE < KERNEL_VERSION(2,3,0)) 
   do_munmap((unsigned long) memory,req_size);
#elif ((LINUX_VERSION_CODE > KERNEL_VERSION(2,4,0)) &&\
     LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)) 
   do_munmap(current->mm,(unsigned long) memory,req_size);
#else
#error "Unable to Handle Dev kernel versions, or versions outside the 2.2 and 2.4 series"
#endif

   return;
}


#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,3,0)) 
int copy_strings(int argc,char ** argv, struct linux_binprm *bprm) 
{
	while (argc-- > 0) {
		char *str;
		int len;
		unsigned long pos;

		if (get_user(str, argv+argc) || !str || !(len = strnlen_user(str, bprm->p))) 
			return -EFAULT;
		if (bprm->p < len) 
			return -E2BIG; 

		bprm->p -= len;
		/* XXX: add architecture specific overflow check here. */ 

		pos = bprm->p;
		while (len > 0) {
			char *kaddr;
			int i, new, err;
			struct page *page;
			int offset, bytes_to_copy;

			offset = pos % PAGE_SIZE;
			i = pos/PAGE_SIZE;
			page = bprm->page[i];
			new = 0;
			if (!page) {
				page = alloc_page(GFP_HIGHUSER);
				bprm->page[i] = page;
				if (!page)
					return -ENOMEM;
				new = 1;
			}
			kaddr = kmap(page);

			if (new && offset)
				memset(kaddr, 0, offset);
			bytes_to_copy = PAGE_SIZE - offset;
			if (bytes_to_copy > len) {
				bytes_to_copy = len;
				if (new)
					memset(kaddr+offset+len, 0, PAGE_SIZE-offset-len);
			}
			err = copy_from_user(kaddr + offset, str, bytes_to_copy);
			kunmap(page);

			if (err)
				return -EFAULT; 

			pos += bytes_to_copy;
			str += bytes_to_copy;
			len -= bytes_to_copy;
		}
	}
	return 0;
}
#endif
