#define __KERNEL__
/* Saint Jude, Linux Kernel Module.
 * Verion: 0.20
 * 
 * July 28, 2001 
 *
 *
 *    Copyright (C) 2001  Timothy Lalwess (lawless@netdoor.com)
 *
 *    This program is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU General Public License as
 *    published by the Free Software Foundation; either version 2 of
 *    the License, or (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be
 *    useful, but WITHOUT ANY WARRANTY; without even the implied
 *    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *    PURPOSE.  See the GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 *    USA.
 *      
 *
 * 
 */

#include <linux/modversions.h>
#include <linux/sys.h>
#include <linux/sched.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <asm/uaccess.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <asm/segment.h>
#include <asm/unistd.h>
#include <linux/mm.h>
#include <linux/smp.h>
#include <linux/signal.h>
#include <linux/slab.h>
#include <asm/unistd.h>
#include <asm/current.h>
#include <sys/syscall.h>
#include <asm/errno.h>
#include <asm/ptrace.h>
#include <asm/pgtable.h>
#include "StJude_lkm.h"


extern SJ_PRIV *sj_priv_hash[SJ_PRIV_HASH];
extern SJ_MEMORY *sj_argv_memory_hash[SJ_MEMORY_HASH];

extern SJ_RULEBASE sj_rulebase[];
                        
/*
 * 
 * On a fork we will need to check if the process has privilege.
 * If yes, then does it have a privilege record?
 *
 *    If it has privilege and no privilege record, then we shall
 *    presume it is a setuid program. At this point we generate
 *    get the index for its restriction, and generate a privilege 
 *    record.
 *
 * If we do have a privilege record, then we need to generate a
 * generate a privilege record for our guy based on the parent's
 * information. 
 *
 */
                       
int
sj_fork (struct pt_regs regs)
{
  int sj_fork_return;
  struct sj_argv_memory  * memory;  

#ifdef __SMP__
read_lock(&sj_running);
read_lock(&tasklist_lock);
#endif

#ifdef DEBUG
printk(" -----------------------------SYSCALL BY %d------------------------\n",current->pid);
#endif

#ifdef __SMP__
read_unlock(&tasklist_lock);
#endif

  sj_fork_return = (*orig_fork) (regs);
 
#ifdef __SMP__
read_lock(&tasklist_lock);
#endif

#ifdef __SMP__
read_lock(&argv_memory_lock);
#endif

memory = get_argv_memory(current);
if (memory)
   {
          
         memory = create_argv_memory(sj_fork_return,memory->argv);
           if (!memory)
             printk("(StJude) sj_fork: Something bad happened, I can't remember argv's.\n");
   }
   else
    printk("(StJude) sj_fork: Can't Recall argv for %d.\n",current->pid);

#ifdef __SMP__
read_unlock(&argv_memory_lock);
write_lock(&priv_lock);
#endif

if (!(current->euid && current->uid && current->suid))
       { 
      struct sj_priv *child;
      struct sj_priv *parent;
      
      parent = get_priv_record(current->pid);
      if (!parent)
	{
#ifdef __SMP__
           write_lock(&argv_memory_lock);
#endif
           suid_hack(current);
#ifdef __SMP__
           write_unlock(&argv_memory_lock);
#endif

           parent = get_priv_record(current->pid);
	   if (!parent)
		{
		   printk("(StJude) sj_fork: Brace yourself, we are going to crash!\n");
                }
        }

   if (sj_fork_return > 0) {
   
          if (parent)
               child = create_priv_record(sj_fork_return,parent->restriction->r_index); 
           else
	       child = create_priv_record(sj_fork_return,0);

       }
  }

#ifdef __SMP__
write_unlock(&priv_lock);
read_unlock(&tasklist_lock);
read_unlock(&sj_running);
#endif


  return sj_fork_return;
}
