#ifdef FSCHECK
#define __KERNEL__
/* Saint Jude, Linux Kernel Module.
 * Verion: 0.20
 * 
 * July 28, 2001 
 *
 *
 *    Copyright (C) 2001  Timothy Lalwess (lawless@netdoor.com)
 *
 *    This program is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU General Public License as
 *    published by the Free Software Foundation; either version 2 of
 *    the License, or (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be
 *    useful, but WITHOUT ANY WARRANTY; without even the implied
 *    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *    PURPOSE.  See the GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 *    USA.
 *      
 *
 * 
 */

#include <linux/modversions.h> 
#include <linux/sys.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <linux/smp.h>
#include <linux/slab.h>
#include <asm/unistd.h>
#include <asm/current.h>
#include <sys/syscall.h>
#include <asm/errno.h>
#include <linux/reboot.h>
#include <linux/smp_lock.h>
#include <linux/fs.h>
#include <linux/sched.h>
#include "StJude_lkm.h"
#include "StJude_string_util.h"
#include "md5.h"



struct fscheck_record ext2_fscheck_record;
struct fscheck_record proc_fscheck_record;
md5_byte_t ext2_fscheck_digest[16];
md5_byte_t proc_fscheck_digest[16];


int init_fscheck_record( struct fscheck_record * rec, char * filename, char * dirname )
{

   int fd = -1;
   char * us_name;
   int size;
   md5_state_t state;
  
   rec->dir_operations = kmalloc(sizeof(struct file_operations), GFP_KERNEL);
   rec->file_operations = kmalloc(sizeof(struct file_operations), GFP_KERNEL);
   rec->super_operations = kmalloc(sizeof(struct super_operations), GFP_KERNEL);
   rec->dentry_operations = kmalloc(sizeof(struct dentry_operations), GFP_KERNEL);
   if ( ! (rec->dir_operations && rec->file_operations && rec->super_operations && rec->dentry_operations ) )
      {
file_init_failure:
          if (rec->dir_operations) kfree(rec->dir_operations);
          if (rec->file_operations) kfree(rec->file_operations);
          if (rec->super_operations) kfree(rec->super_operations);
          if (rec->dentry_operations) kfree(rec->dentry_operations);
          printk("STMICHAEL: FAILED TO INIT FILECHECKS\n");
          return -1;
      }

   // First, the File Data 
   size = sj_strlen(filename)+1;
   us_name = sj_malloc(size);
   copy_to_user(us_name,filename,size);
   fd = sj_open(us_name, O_RDONLY, 0);
   sj_free(us_name,size);

   if (fd < 0) {
      printk("STMICHAEL: FAILED TO INIT FILECHECKS\n", fd);
      goto file_init_failure; 
   }


   sj_memcpy(rec->file_operations, current->files->fd[fd]->f_op,
              sizeof(struct file_operations));

   md5_init(&state);
   md5_append(&state, (const md5_byte_t *) current->files->fd[fd]->f_op, 
             sizeof(struct file_operations));
   md5_finish(&state, rec->file_operations_digest);


   sj_memcpy(rec->dentry_operations, current->files->fd[fd]->f_dentry->d_op,
              sizeof(struct dentry_operations));

   md5_init(&state);
   md5_append(&state, (const md5_byte_t *) current->files->fd[fd]->f_dentry->d_op, 
              sizeof(struct dentry_operations));
   md5_finish(&state, rec->dentry_operations_digest);

   sj_memcpy(rec->super_operations, current->files->fd[fd]->f_dentry->d_sb->s_op,
             sizeof(struct super_operations));

   md5_init(&state);
   md5_append(&state, (const md5_byte_t *) current->files->fd[fd]->f_dentry->d_sb->s_op, 
              sizeof(struct super_operations));
   md5_finish(&state, rec->super_operations_digest);

   sj_close(fd); fd = -1;
  
   size = sj_strlen(dirname)+1;
   us_name = sj_malloc(size);
   copy_to_user(us_name,dirname, size);
   fd = sj_open(us_name, O_RDONLY | O_DIRECTORY, 0);
   sj_free(us_name,size);

   if (fd < 0) {
      printk("STMICHAEL: FAILED TO INIT FILECHECKS\n");
      goto file_init_failure; 
   }
   
   sj_memcpy(rec->dir_operations, current->files->fd[fd]->f_op,
             sizeof(struct file_operations));
   md5_init(&state);
   md5_append(&state, (const md5_byte_t *) current->files->fd[fd]->f_op, sizeof(struct file_operations));
   md5_finish(&state, rec->directory_operations_digest);
#ifdef DEBUG 
   { int i; printk("Digest: "); for (i = 0; i < 16; i++) printk("%X", rec->directory_operations_digest[i]); printk("\n"); }
#endif
   sj_close(fd); fd = -1;


   return 0;
   
}


void check_fscheck_record( struct fscheck_record * rec, char * filename, char * dirname )
{
   md5_state_t state;
   md5_byte_t digest[16];
   int i;
   int fd = -1;
   int size;
   char * us_name;

   // First, Dir options.
   size = sj_strlen(filename)+1;
   us_name = sj_malloc(size);
   if (!us_name)
      { 
       printk("NO MEM\n");
       return;
      }
   copy_to_user(us_name,filename,size);
   for (i = 1; i < 100000; i++);
   fd = sj_open(us_name, O_RDONLY, 0);
   sj_free(us_name,size);

   if (fd < 0)
      {   
          printk("(STMICHAEL) File to be Checked is Missing. Possible Evasion or Probe of StMichael. (%s)(%d)", filename,fd); 
          return;
      }
   
   md5_init(&state);
   md5_append(&state, (const md5_byte_t *) current->files->fd[fd]->f_op, 
                      sizeof(struct file_operations));
   md5_finish(&state, digest);
   
   for (i = 0; i < 16; i++)
      if (digest[i] != rec->file_operations_digest[i])
         {
           sj_print("(STMICHAEL) File Opperations Table Modified. Possible Kernel Rootkit. Attempting to Restore.\n");
           sj_memcpy( current->files->fd[fd]->f_op,
                     rec->file_operations,
                     sizeof(struct file_operations));
         }
   
   md5_init(&state);
   md5_append(&state, (const md5_byte_t *) current->files->fd[fd]->f_dentry->d_op, 
              sizeof(struct dentry_operations));
   md5_finish(&state, digest);

   for (i = 0; i < 16; i++)
      if (digest[i] != rec->dentry_operations_digest[i])
         {
           sj_print("(STMICHAEL) Dentry Opperations Table Modified. Possible Kernel Rootkit. Attempting to Restore.\n");
           sj_memcpy( current->files->fd[fd]->f_dentry->d_op,
                      rec->dentry_operations,
                     sizeof(struct dentry_operations));
         }
   
   md5_init(&state);
   md5_append(&state, (const md5_byte_t *) current->files->fd[fd]->f_dentry->d_sb->s_op, 
                     sizeof(struct super_operations));
   md5_finish(&state, digest);
   
   for (i = 0; i < 16; i++)
      if (digest[i] != rec->super_operations_digest[i])
         {
           sj_print("(STMICHAEL) File Opperations Table Modified. Possible Kernel Rootkit. Attempting to Restore.\n");
           sj_memcpy( current->files->fd[fd]->f_dentry->d_sb->s_op,
                     rec->super_operations,
                     sizeof(struct super_operations));
         }
   
   sj_close(fd); fd = -1;

   size = sj_strlen(dirname)+1;
   us_name = sj_malloc(size);
   if (!us_name)
      { 
       printk("NO MEM\n");
       return;
      }
   copy_to_user(us_name,dirname,size);
   fd = sj_open(us_name, O_RDONLY, 0);
   sj_free(us_name,size);

   if (fd < 0)
      {   
          printk("(STMICHAEL) Directory to be Checked is Missing. Possible Evasion or Probe of StMichael. (%s)\n\n", dirname); 
          return;
      }

   md5_init(&state);
   md5_append(&state, (const md5_byte_t *) current->files->fd[fd]->f_op, 
              sizeof(struct file_operations));
   md5_finish(&state, digest);

#ifdef DEBUG
   { int i; printk("Digest: "); for (i = 0; i < 16; i++) printk("%X", rec->directory_operations_digest[i]); printk("\n"); }
   { int i; printk("Digest: "); for (i = 0; i < 16; i++) printk("%X", digest[i]); printk("\n"); }
#endif

   for (i = 0; i < 16; i++)
      if (digest[i] != rec->directory_operations_digest[i])
         {
           sj_print("(STMICHAEL) Directory Opperations Table Modified. Possible Kernel Rootkit. Attempting to Restore.\n");
           sj_memcpy( current->files->fd[fd]->f_op,
                      rec->dir_operations,
                     sizeof(struct file_operations));
         }



   
   sj_close(fd); fd = -1;
   
   return; 
}

int init_fscheck_records( void )
{
   md5_state_t state;
    
   if (init_fscheck_record( &ext2_fscheck_record, "/sbin/init" , "/"))
           return -1;
   if (init_fscheck_record( &proc_fscheck_record, "/proc/ksyms", "/proc" ))
           return -1;
    
   md5_init(&state);
   md5_append(&state, (const md5_byte_t *) &ext2_fscheck_record, sizeof(struct fscheck_record));
   md5_finish(&state, ext2_fscheck_digest);
   md5_init(&state);
   md5_append(&state, (const md5_byte_t *) &proc_fscheck_record, sizeof(struct fscheck_record));
   md5_finish(&state, proc_fscheck_digest);

   return 0;

}

void check_fscheck_records( void )
{
   md5_state_t state;
   md5_byte_t digest[16];
   int i;

   md5_init(&state);
   md5_append(&state, (const md5_byte_t *) &ext2_fscheck_record, sizeof(struct fscheck_record));
   md5_finish(&state, digest);

   for (i = 0; i < 16; i++)
      if (digest[i] != ext2_fscheck_digest[i])
         {
           sj_print("(STMICHAEL) EXT2 Filesystem check table corrupted. Non-Recoverable Error.\n");
           danger_will_robinson_danger();
           return;
         }

   check_fscheck_record( &ext2_fscheck_record, "/sbin/init" , "/");
   

   md5_init(&state);
   md5_append(&state, (const md5_byte_t *) &proc_fscheck_record, sizeof(struct fscheck_record));
   md5_finish(&state, digest);
   
    for (i = 0; i < 16; i++)
       if (digest[i] != proc_fscheck_digest[i])
         {
           sj_print("(STMICHAEL) PROC Filesystem check table corrupted. Non-Recoverable Error.\n");
           danger_will_robinson_danger();
           return;
         }
   
    check_fscheck_record( &proc_fscheck_record, "/proc/ksyms", "/proc" );

    return;
}

#endif
