#define __KERNEL__
/* Saint Jude, Linux Kernel Module.
 * Verion: 0.20
 * 
 * July 28, 2001 
 *
 *
 *    Copyright (C) 2001  Timothy Lalwess (lawless@netdoor.com)
 *
 *    This program is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU General Public License as
 *    published by the Free Software Foundation; either version 2 of
 *    the License, or (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be
 *    useful, but WITHOUT ANY WARRANTY; without even the implied
 *    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *    PURPOSE.  See the GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 *    USA.
 *      
 *
 * 
 */

#include <linux/modversions.h> 
#include <linux/sys.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <linux/smp.h>
#include <linux/slab.h>
#include <asm/unistd.h>
#include <asm/current.h>
#include <sys/syscall.h>
#include <asm/errno.h>
#include <linux/reboot.h>
#include <linux/smp_lock.h>
#include "StJude_lkm.h"
#include "StJude_string_util.h"


#ifdef CLOAK
struct mli *iml;
struct module **eml;
#endif

SM_INTEGRITY_RECORD  recorded_sys_call_table[NR_syscalls];

#ifdef USE_CHECKSUM

SM_INTEGRITY_RECORD recorded_dependency_table[NR_sj_dependencies];

int infractions = 0;

md5_byte_t dependency_md5[16];
md5_byte_t syscall_md5[16];

void sj_dependency_table_init( void )
{


  int i = 0;

  recorded_dependency_table[i++].orig_call = (void *) kmalloc;
  recorded_dependency_table[i++].orig_call = (void *) kfree;
#define __SM_PRINTK 2
  recorded_dependency_table[i++].orig_call = (void *) printk;
#define __SM_SCHEDULE 3
  recorded_dependency_table[i++].orig_call = (void *) schedule;
#define __SM_SYNC 4
  recorded_dependency_table[i++].orig_call = (void *) syscall_sync;
#define __SM_REBOOT 5 
  recorded_dependency_table[i++].orig_call = (void *) syscall_reboot;
  recorded_dependency_table[i++].orig_call = (void *) schedule_task;
  recorded_dependency_table[i++].orig_call = (void *) panic;
  recorded_dependency_table[i++].orig_call = (void *) machine_restart;
  recorded_dependency_table[i++].orig_call = (void *) machine_halt;
  recorded_dependency_table[i++].orig_call = (void *) machine_power_off;
  recorded_dependency_table[i++].orig_call = (void *) do_execve;
#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,4,0))
  recorded_dependency_table[i++].orig_call = (void *) kernel_read;
  recorded_dependency_table[i++].orig_call = (void *) open_exec;
#endif
  recorded_dependency_table[i++].orig_call = (void *) strnicmp;

  return;
}

void sj_integrity_check_checksum_init(SM_INTEGRITY_RECORD *record, void * target_function )
{

  md5_state_t state;
#ifdef USE_SHA1
  SHA1_CTX context;
#endif

  record->orig_call = target_function;
  
  md5_init(&state);
  md5_append(&state, (const md5_byte_t *) target_function,
             md5_sample_len );
  md5_finish(&state, record->recorded_md5_digest);

#ifdef USE_SHA1
  SHA1Init(&context);
  SHA1Update(&context,(unsigned char *) target_function, sha1_sample_len);
  SHA1Final(record->recorded_sha1_digest,&context);
#endif

  return;
  
}

#endif

void sj_integrity_check_init( void )
{

 struct module *m;

 register int i;

 
 // First, Syscalls.
 for ( i = 0; i < NR_syscalls; i++ )
 {

#ifdef USE_CHECKSUM

   sj_integrity_check_checksum_init( recorded_sys_call_table + i, sys_call_table[i] );

#else

   recorded_sys_call_table[i] = sys_call_table[i];

#endif

 }

#ifdef CLOAK

 for ( m = *eml; m != NULL; m = m->next )
 {
   struct mli *k;
   k = kmalloc(sizeof(struct mli),GFP_KERNEL);
   k->module = m;
   k->namelen =  m->size - sizeof(struct module);
   k->namelen =  k->namelen > 0 ? k->namelen : 0;
   k->name = kmalloc(k->namelen + 1,GFP_KERNEL);
   sj_memcpy(k->name,m->name,k->namelen); 
   k->name[k->namelen]  = '\0';

   if (iml)
       iml->next = k;
   else
       iml = k;

   }
#endif

// Then, if we are working with Checksums.. We do the Dependencies, and the Tables themselves.
    
#ifdef USE_CHECKSUM
 
 sj_dependency_table_init();
 for ( i = 0; i < NR_sj_dependencies; i++ )
    if( recorded_dependency_table[i].orig_call != NULL)
        { 
          sj_integrity_check_checksum_init( &recorded_dependency_table[i], 
                                recorded_dependency_table[i].orig_call );
        }


 {
   md5_state_t state;

   md5_init(&state);
   md5_append(&state, (const md5_byte_t *) recorded_sys_call_table,
            (sizeof(SM_INTEGRITY_RECORD) * NR_syscalls));
   md5_finish(&state, syscall_md5);


   md5_init(&state);
   md5_append(&state, (const md5_byte_t *) recorded_dependency_table,
            (sizeof(SM_INTEGRITY_RECORD) * NR_sj_dependencies) );
   md5_finish(&state, dependency_md5);

 }

#endif

} 

#ifdef USE_CHECKSUM
int
sj_check_dependency_table()
{
  
  md5_state_t state;
  md5_byte_t digest[16];
  register int j;

   md5_init(&state);
   md5_append(&state, (const md5_byte_t *) recorded_dependency_table,
            (sizeof(SM_INTEGRITY_RECORD) * NR_sj_dependencies) );
   md5_finish(&state, digest);

    for (j = 0; j < 16; j++)
        if ( digest[j] != dependency_md5[j] )
                     return 1;
  
   return 0;
}

int
sj_check_sys_call_table()
{
  
  md5_state_t state;
  md5_byte_t digest[16];
  register int j;


   md5_init(&state);
   md5_append(&state, (const md5_byte_t *) recorded_sys_call_table, 
          (sizeof(SM_INTEGRITY_RECORD) * NR_syscalls));

   md5_finish(&state, digest);

    for (j = 0; j < 16; j++)
        if ( digest[j] != syscall_md5[j] )
                     return 1;
  
   return 0;
}

/* 1 everything is ok
   0 its bad
 */
int
sj_check_specific_checksum(SM_INTEGRITY_RECORD *record)
{

  md5_state_t state;
  md5_byte_t md5_digest[16];
#ifdef USE_SHA1
  unsigned char sha1_digest[20];
  SHA1_CTX context;
#endif

  register int j;


   md5_init(&state);
   md5_append(&state, (const md5_byte_t *) record->orig_call, 
          md5_sample_len);

   md5_finish(&state, md5_digest);

    for (j = 0; j < 16; j++)
        if ( md5_digest[j] != record->recorded_md5_digest[j] )
                     return 0;
#ifdef USE_SHA1
    SHA1Init(&context);
    SHA1Update(&context,(unsigned char *) record->orig_call, sha1_sample_len);
    SHA1Final(sha1_digest,&context);
   
    
    for (j = 0; j < 20; j++)
    { 
       if ( sha1_digest[j] != record->recorded_sha1_digest[j] )
                     return 0; 
    } 
#endif

   return 1;
}

void
sj_check_dependency_integrity ( void )
{


   register int i;

   if (sj_check_dependency_table())
           { 
#ifdef ROBINSON
             danger_will_robinson_danger();
#endif
                  return;
           }

   for (i = 0; i < NR_sj_dependencies; i++)
    {
       
       if (!sj_check_specific_checksum( recorded_dependency_table + i) )
             {
#ifdef ROBINSON
                  danger_will_robinson_danger();
#endif
                  return;
             } 
    }
       
        return;
            
}   
               

void danger_will_robinson_danger ( void )
{

// When in Fear, When In Doubt, Run Around, Scream and Shout!

    if (sj_check_specific_checksum( recorded_dependency_table + __SM_PRINTK))
                {
                  sj_print("(STMICHAEL) Non-Recoverable Attack Detected: The Kernel was modified directly.\n");

                  if (sj_check_specific_checksum( recorded_dependency_table + __SM_SYNC))
                  {
                      (*syscall_sync)();
                  }
                  if (sj_check_specific_checksum( recorded_dependency_table + __SM_SCHEDULE))
                   {
			schedule();
			schedule();
			schedule();
                   }
              
                }
               if (sj_check_specific_checksum( recorded_dependency_table + __SM_REBOOT))
                  {
                      sj_print("(STMICHAEL) Rebooting\n");
                      machine_restart(NULL);
                  }

                    // Sit, and Spin.
                    lock_kernel();
                    while(1) 
                         {
				;
                         }
}
               
#endif




void
sj_check_sys_call_integrity ( void  )
{
   register int i;
#if !defined(USE_CHECKSUM)
   register int j;
#endif


  /* 
     Verify that the syscall table is the same. 
     If its changed then respond 

   */

#ifdef USE_CHECKSUM 
   if (sj_check_sys_call_table())
     {
#ifdef ROBINSON
      danger_will_robinson_danger();
#endif
                  return;
     }

#endif 
	

   for (i = 0; i < NR_syscalls; i++)
#ifdef USE_CHECKSUM
    {

       if ( recorded_sys_call_table[i].orig_call != sys_call_table[i] )
#else
       if ( recorded_sys_call_table[i] != sys_call_table[i] )
#endif
          {

	    sj_print("(STMICHAEL) A Systemcall Table Change was detected. Possible Rootkit Present\n");
            sj_print("(STMICHAEL) Returning SysCall Table to Pristine State.\n");

#ifdef USE_CHECKSUM
                  sys_call_table[i] = recorded_sys_call_table[i].orig_call;
#else
            for ( j = 0; j < NR_syscalls; j++)
                  sys_call_table[j] = recorded_sys_call_table[j];

            break;
#endif
    
#ifdef USE_CHECKSUM
          }

       if ( !sj_check_specific_checksum( recorded_sys_call_table + i ) )
	    { 
#ifdef ROBINSON
                  danger_will_robinson_danger();
#endif
                  return;
               
            }
       
#endif           
            
      }   

  return;
}

#ifdef CLOAK

void sj_add_module_list( void )
{

 // When Modules are added, they are added to the head. 

   struct mli *k;

   k = kmalloc(sizeof(struct mli),GFP_KERNEL);
   k->module = *eml;
   k->namelen = (*eml)->size - sizeof(struct module);
   k->namelen =  k->namelen > 0 ? k->namelen : 0;
   k->name = kmalloc(k->namelen + 1,GFP_KERNEL);
   sj_memcpy(k->name,(*eml)->name,k->namelen); 
   k->name[k->namelen]  = '\0';

   k->next = iml;
   iml = k;
   return;
}

int  sj_check_module_list( void )
{
  struct mli *k;
  struct module *m;
  struct module *p = NULL;

  for(m = *eml, k = iml; k != NULL && m != NULL; k=k->next,m=m->next )
  {
    if (k->module !=  m)
      {
         if (k->next && k->next->module == m)
         {
            sj_print("(STMICHAEL) Possibe LKM RootKit Detected.\n"); 
            sj_print("(STMICHAEL) Detected another module being hidden. Restoring.\n");
            sj_print("(STMICHAEL) Module Name: %s\n", k->name);

            if(p)
               {
                  p->next = k->module;
               }
            else
               {
                  *eml = k->module;
               }
               k->module->next = m;
#ifdef ATTEMPT_FORCEFULL_UNLOAD
           if(!orig_delete_module(k->module->name))
               sj_delete_module(k->module->name)
#endif            

           return 1;
         } 
         else
         { 
            struct module *a;
            struct mli *b;
            
            sj_print("(STMICHAEL) Possibe LKM RootKit Detected. \n");
            sj_print("(STMICHAEL) Detected multiple modules being hidden. Trying Restore.\n");
           // If other things are happening, then this could be dangerous ;)  
           *eml = iml->module;
           for(a = *eml, b = iml; b != NULL; b=b->next, a=a->next)
             a->next = b->next->module;      
      
           return -1;
         }
 
       }
      
       p = m;
  }

  return 0;
}

void sj_remove_module_list( const char * name )
{
   struct mli *k = NULL;
   struct mli *m = NULL;
   struct module *n = NULL;

  // Modules can Be deleted from anywhere within the structure.
  if (!name)
      goto handle_autoclean;

  for ( k = iml; k->next != NULL ; k = k->next )
  {
     if (!strncmp(k->name,name,k->namelen))
     {
       // match

       if (m)
          m->next = k->next;
       else
          iml = k->next;
 
       kfree(k->name);
       kfree(k);
       return;

     }
     m = k;
     
  }

  // Should Never Reach this point. If we do.. bitch.

  printk("Tried to Delete a Module that Doesn't exist in SM despite a positive return from delete_module.\n");
 
  return; 

handle_autoclean:

  for(n = *eml, k = iml; k != NULL && n != NULL; k=k->next,n=n->next )
    {
      if ( k && (k->module != n))
       {
        if (k->name) {
           sj_remove_module_list( k->name );
           goto handle_autoclean;
           }
       }
    }
}

#endif


#ifdef ROKMEM

ssize_t sj_deny_write ( struct file * file, const char * stuff, size_t size, loff_t * loff)
{
  return 0;
}

void sj_kemem_ro ( void )
{

   int fd = -1;
   char * us_name;
   int size;
   
   size = sj_strlen("/dev/kmem")+1;
   us_name = (char *) sj_malloc(size);
   copy_to_user(us_name,"/dev/kmem",size);
   fd = sj_open(us_name, O_RDONLY, 0);
   sj_free(us_name,size);

   if (fd < 0) {
      printk("STMICHAEL: FAILED TO OPEN KMEM\n", fd);
      return;
   }

   sj_write_kmem = current->files->fd[fd]->f_op->write;
   current->files->fd[fd]->f_op->write = sj_deny_write;

   sj_close(fd); fd = -1;
  
   return;
   
}

void sj_kemem_rw ( void )
{

   int fd = -1;
   char * us_name;
   int size;
   
   size = sj_strlen("/dev/kmem")+1;
   us_name = (char *) sj_malloc(size);
   copy_to_user(us_name,"/dev/kmem",size);
   fd = sj_open(us_name, O_RDONLY, 0);
   sj_free(us_name,size);

   if (fd < 0) {
      printk("STMICHAEL: FAILED TO OPEN KMEM\n", fd);
      return;
   }

   current->files->fd[fd]->f_op->write = sj_write_kmem;
   sj_write_kmem = NULL;

   sj_close(fd); fd = -1;
  
   return;
   
}

#endif 
